#include <cstdlib>
#include <iostream>
#include <map>
#include <string>
#include <vector>
#include <rdbxx/RDB.h>

void init_lookup( std::map<std::string, std::vector<int> >& lookup, const int n ) {
  typedef std::map<std::string, std::vector<int> >::value_type vt;

  lookup.insert( vt( "n",      std::vector<int>(n) ) );
  lookup.insert( vt( "sum",    std::vector<int>(n) ) );
  lookup.insert( vt( "ave",    std::vector<int>(n) ) );
  lookup.insert( vt( "std",    std::vector<int>(n) ) );
  lookup.insert( vt( "min",    std::vector<int>(n) ) );
  lookup.insert( vt( "max",    std::vector<int>(n) ) );
  lookup.insert( vt( "rss",    std::vector<int>(n) ) );
  lookup.insert( vt( "fq",     std::vector<int>(n) ) );
  lookup.insert( vt( "median", std::vector<int>(n) ) );
  lookup.insert( vt( "lq",     std::vector<int>(n) ) );
  lookup.insert( vt( "pN",     std::vector<int>(n) ) );
}

// Using group columns, find the min and max values as well as the
// number of elements in a group.
int main( int argc, char* argv[] ) {

  try {
    // Set up input and output files...
    RDB irdb( &std::cin );
    RDB ordb( &std::cout );

    // Get the number of input columns...
    size_t nColumns       = irdb.nColumns( );

    // Set up index lookups for the output columns...
    std::map<std::string, std::vector<int> > lookup;
    init_lookup( lookup, nColumns );

    // Foreach input column...
    for ( size_t idx = 0; idx < nColumns; idx++ ) {
      std::string name = irdb.getName( idx );
      std::string def  = irdb.getDef(  idx );

      // If its numeric, setup the min, max, and count column...
      if ( RDBColumn::NUMERIC == irdb.getType( idx ) ) {
	// Create statistics columns foreach numeric input column...
	ordb.setColumn( name + "_n",      def );
	ordb.setColumn( name + "_sum",    def );
	ordb.setColumn( name + "_ave",    def );
	ordb.setColumn( name + "_std",    def );
	ordb.setColumn( name + "_min",    def );
	ordb.setColumn( name + "_max",    def );
	ordb.setColumn( name + "_rss",    def );
	ordb.setColumn( name + "_fq",     def );
	ordb.setColumn( name + "_median", def );
	ordb.setColumn( name + "_lq",     def );
	ordb.setColumn( name + "_pN",     def );

	lookup["n"][idx]      = ordb.nColumns( ) - 11;
	lookup["sum"][idx]    = ordb.nColumns( ) - 10;
	lookup["ave"][idx]    = ordb.nColumns( ) - 9;
	lookup["std"][idx]    = ordb.nColumns( ) - 8;
	lookup["min"][idx]    = ordb.nColumns( ) - 7;
	lookup["max"][idx]    = ordb.nColumns( ) - 6;
	lookup["rss"][idx]    = ordb.nColumns( ) - 5;
	lookup["fq"][idx]     = ordb.nColumns( ) - 4;
	lookup["median"][idx] = ordb.nColumns( ) - 3;
	lookup["lq"][idx]     = ordb.nColumns( ) - 2;
	lookup["pN"][idx]     = ordb.nColumns( ) - 1;
	
      // Else assume its a break column...
      } else {
	// Set up a group on any non-numeric field...
	irdb.setGroup( idx );
	ordb.setColumn( irdb.getColumn( idx ) );
	
      }
    }
    
    int stat;
    // Read the input...
    while ( stat = irdb.read( ) ) {

      // Beginning of a group...
      if ( RDB::RBOG & stat ) {
	for ( size_t idx = 0; idx < nColumns; idx++ ) {
	  if ( RDBColumn::NUMERIC == irdb.getType( idx ) ) {
	    ordb.setData( lookup["n"][idx],      1.0 );
	    ordb.setData( lookup["sum"][idx],    irdb.getDataDouble( idx ) );
	    ordb.setData( lookup["ave"][idx],    irdb.getDataDouble( idx ) );
	    ordb.setData( lookup["std"][idx],    "Inf" );
	    ordb.setData( lookup["min"][idx],    irdb.getDataDouble( idx ) );
	    ordb.setData( lookup["max"][idx],    irdb.getDataDouble( idx ) );
	    ordb.setData( lookup["rss"][idx],    "Inf" );
	    ordb.setData( lookup["fq"][idx],     "Inf" );
	    ordb.setData( lookup["median"][idx], "Inf" );
	    ordb.setData( lookup["lq"][idx],     "Inf" );
	    ordb.setData( lookup["pN"][idx],     "Inf" );

	  }
	}
      // Middle of a group...
      } else {
	for ( size_t idx = 0; idx < nColumns; idx++ ) {
	  double val;
	  // Keep accumulating stats...
	  if ( RDBColumn::NUMERIC == irdb.getType( idx ) ) {
	    ordb.setData( lookup["n"][idx], ordb.getDataDouble( lookup["n"][idx] ) + 1.0 );
	    //ordb.setData( lookup["sum"][idx], ordb.getDataDouble( lookup["sum"][idx] ) + irdb.getDataDouble( idx ) );
	    ordb.setData( lookup["ave"][idx], ordb.getDataDouble( lookup["ave"][idx] ) + irdb.getDataDouble( idx ) );
	    //ordb.setData( lookup["std"][idx],    "Inf" );
	    if ( ordb.getDataDouble( lookup["min"][idx] ) > (val = irdb.getDataDouble( idx )) )
	      ordb.setData( lookup["min"][idx], val );
	    if ( ordb.getDataDouble( lookup["max"][idx] ) < (val = irdb.getDataDouble( idx )) )
	      ordb.setData( lookup["max"][idx], val );
	    //ordb.setData( lookup["rss"][idx],    "Inf" );
	    //ordb.setData( lookup["fq"][idx],     "Inf" );
	    //ordb.setData( lookup["median"][idx], "Inf" );
	    //ordb.setData( lookup["lq"][idx],     "Inf" );
	    //ordb.setData( lookup["pN"][idx],     "Inf" );

	  }
	}
      }

      // End of a group...
      if ( RDB::REOG & stat ) {
	for ( size_t idx = 0; idx < nColumns; idx++ ) {
	  ordb.setData( lookup["ave"][idx], ordb.getDataDouble( lookup["sum"][idx] ) / ordb.getDataDouble( lookup["n"][idx] ) );

	}

	ordb.write( );

      }
    }
  } catch ( RDBErr& re ) {
    std::cerr << "RDBErr:" << std::endl << re << std::endl;
    return EXIT_FAILURE;
    
  } catch ( Exception& E ) {
    std::cerr << "Exception:" << std::endl << E << std::endl;
    return EXIT_FAILURE;

  } catch ( std::exception& e ) {
    std::cerr << "std::exception:" << std::endl << e.what( ) << std::endl;
    return EXIT_FAILURE;
    
  } catch ( ... ) {
    std::cerr << "Caught something unexpected..." << std::endl;
    return EXIT_FAILURE;

  }

  return EXIT_SUCCESS;
}
