#define CATCH_CONFIG_MAIN
#include <catch2/catch.hpp>

#include <rdbxx/RDBColumnTmplt.h>
#include <rdbxx/RDBErr.h>

using namespace std;

TEST_CASE( "RDBColumn Header Test Suite" ) {
  
  string name( "my column name" );
  string def( "15N< What description?" );
  long width = 15;
  RDBColumn::Type type = RDBColumn::NUMERIC;
  RDBColumn::Just just = RDBColumn::LEFT;
  string desc( "What description?" );  
  RDBStringColumn c1( name, def );

  REQUIRE(  c1.getName( ) ==  name );
  REQUIRE(   c1.getDef( ) ==   def );
  REQUIRE( c1.getWidth( ) == width );
  REQUIRE(  c1.getType( ) ==  type );
  REQUIRE(  c1.getJust( ) ==  just );
  REQUIRE(  c1.getDesc( ) ==  desc );

  def.replace( def.find( 'N' ), 1, 1, 'S' );
  type = RDBColumn::STRING;
  c1.setDef( def );
  REQUIRE(  c1.getName( ) ==  name );
  REQUIRE(   c1.getDef( ) ==   def );
  REQUIRE( c1.getWidth( ) == width );
  REQUIRE(  c1.getType( ) ==  type );
  REQUIRE(  c1.getJust( ) ==  just );
  REQUIRE(  c1.getDesc( ) ==  desc );

  def.replace( def.find( 'S' ), 1, 1, 'M' );
  type = RDBColumn::MONTH;
  c1.setDef( def );
  REQUIRE(  c1.getName( ) ==  name );
  REQUIRE(   c1.getDef( ) ==   def );
  REQUIRE( c1.getWidth( ) == width );
  REQUIRE(  c1.getType( ) ==  type );
  REQUIRE(  c1.getJust( ) ==  just );
  REQUIRE(  c1.getDesc( ) ==  desc );

  def.replace( def.find( 'M' ), 1, 1, '-' );
  type = RDBColumn::STRING;
  c1.setDef( def );
  REQUIRE(  c1.getName( ) ==  name );
  REQUIRE(   c1.getDef( ) ==   def );
  REQUIRE( c1.getWidth( ) == width );
  REQUIRE(  c1.getType( ) ==  type );
  REQUIRE(  c1.getJust( ) ==  just );
  REQUIRE(  c1.getDesc( ) ==  desc );

  def   = "N<";
  width = -1;
  type  = RDBColumn::NUMERIC;
  just  = RDBColumn::LEFT;
  desc  = "";
  c1.setDef( def );
  REQUIRE(  c1.getName( ) ==  name );
  REQUIRE(   c1.getDef( ) ==   def );
  REQUIRE( c1.getWidth( ) == width );
  REQUIRE(  c1.getType( ) ==  type );
  REQUIRE(  c1.getJust( ) ==  just );
  REQUIRE(  c1.getDesc( ) ==  desc );

  REQUIRE_THROWS_AS( c1.setDef( "" ), RDBErr );
  REQUIRE_THROWS_AS( c1.setDef( "12<N foo" ), RDBErr );
  REQUIRE_THROWS_AS( c1.setDef( "P" ), RDBErr );
  REQUIRE_THROWS_AS( c1.setDef( "14N-" ), RDBErr );
  REQUIRE_THROWS_AS( c1.setDef( "14N<Description..." ), RDBErr );
  REQUIRE_THROWS_AS( c1.setDef( "14N<\tDescription..." ), RDBErr );

}
