#define CATCH_CONFIG_MAIN
#include <catch2/catch.hpp>

#include <cstdlib>
#include <rdbxx/RDBComment.h>

using namespace std;

void visualtest( RDBComment& c, string& commexp, string& keyexp, string& valexp ) {
  cerr << "RDBComment::getComment( ):  `" << c.getComment( ) << "' == `" << commexp << "'" << endl;
  cerr << "RDBComment::getKeyword( ):  `" << c.getKeyword( ) << "' == `" << keyexp << "'" << endl;
  cerr << "RDBComment::getValue(   ):  `" << c.getValue( )   << "' == `" << valexp << "'" << endl << endl;

}

void autotest( RDBComment& c, string& commexp, string& keyexp, string& valexp ) {
  REQUIRE( c.getCommentText( ) == commexp );
  REQUIRE( c.getKeyword( ) ==  keyexp );
  REQUIRE( c.getValue( ) ==    valexp );

}

void test( RDBComment& c, string& commexp, string& keyexp, string& valexp, bool automatic ) {
  if ( automatic ) 
    autotest(   c, commexp, keyexp, valexp );
  else 
    visualtest( c, commexp, keyexp, valexp );

}

TEST_CASE( "RDBComment Test Suite" ) {

  bool automatic =
      std::getenv("RDB_TEST_MODE")
      ? string( std::getenv("RDB_TEST_MODE") ) != string("visual")
      : true;

  RDBComment c1( "# One plain header comment" );
  string commexp("One plain header comment");
  string keyexp("");
  string valexp("");
  test( c1, commexp, keyexp, valexp, automatic );

  RDBComment c2( " One plain header comment " ); 
  commexp = "One plain header comment ";
  keyexp  = "";
  valexp  = "";
  test( c2, commexp, keyexp, valexp, automatic );

  RDBComment c3( "One plain header comment\t" ); 
  commexp = "One plain header comment\t";
  keyexp  = "";
  valexp  = "";
  test( c3, commexp, keyexp, valexp, automatic );

  RDBComment c4( "#:Header = Variable comment..." );
  commexp = "Header = Variable comment...";
  keyexp  = "Header";
  valexp  = "Variable comment...";
  test( c4, commexp, keyexp, valexp, automatic );

  RDBComment c5( ":Header =Variable comment..." );
  commexp = "Header =Variable comment...";
  keyexp  = "Header";
  valexp  = "Variable comment...";
  test( c5, commexp, keyexp, valexp, automatic );

  RDBComment c6( "Header= Variable comment..." );
  commexp = "Header= Variable comment...";
  keyexp  = "";
  valexp  = "";
  test( c6, commexp, keyexp, valexp, automatic );
}
